/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2008-2009 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * thist file is based on textedit.h (TextEdit demo) from Qt 4.3.2
 * Copyright (C) 2004-2008 Trolltech ASA. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 * */

#ifndef PRINTPREVIEW_H
#define PRINTPREVIEW_H

//#include <QMainWindow>
#include <QDialog>
#include <QAbstractScrollArea>
#include <QPrinter>
#include <QPointF>
#include <QSizeF>

#include <math.h>

class PreviewView;
class QTextDocument;
class QBoxLayout;
class QPushButton;
class QLabel;
class QComboBox;
class QSplitter;

#define DEFAULT_MARGIN 15  // mm
#define DEFAULT_MARGIN_LEFT		15 // 20
#define DEFAULT_MARGIN_RIGHT	10// 10
#define DEFAULT_MARGIN_TOP		10 // 10
#define DEFAULT_MARGIN_BOTTOM	10 // 10
#define DEFAULT_FONT_SIZE	9
//#define SET_BACKGROUND

#define ZOOM_STEP_MULT
#define SIZE_STEP			1.414213562
#define PAGES_CACHED_MAX	32
#define THUMBS_DEF_SCALE	0.125

#define MULTICOLUMN
#define THUMBS_AUTOSCALE

#if (defined (MULTICOLUMN) && !defined (THUMBS_AUTOSCALE))
#define THUMBS_MULTICOLUMN
#endif

#define CACHE_ALL
//#define CACHE_ALWAYS_EN  // always use cache, even in normal mode witch cols=1
//#define CACHE_PIXMAP  // use QPixmap instead QImage
#ifndef CACHE_PIXMAP
#define DRAW_THRU_PIX
#endif

#ifndef QT_NO_DEBUG
//#define PP_DEBUG2
#endif

static inline int inchesToPixelsX(float inches, QPaintDevice *device) { return qRound(inches * device->logicalDpiX()); }
static inline int inchesToPixelsY(float inches, QPaintDevice *device) { return qRound(inches * device->logicalDpiY()); }

static inline float mmToInches(double mm) { return mm*0.039370147; }

	//class PrintPreview : public QMainWindow
class PrintPreview : public QDialog
{
    Q_OBJECT

public:
    PrintPreview(QWidget *parent, const QTextDocument *document = 0);
    virtual ~PrintPreview();

	void setDocument(const QTextDocument *document);

    QSizeF paperSize;
//    QRectF pageRect;
//    QPointF pageTopLeft;
//    static void setDefaults(QTextDocument*, QPrinter*, QWidget*);
    void setDefaults(QTextDocument*, QPrinter*, QWidget*);

private slots:
    void print();
    void pageSetup();
	void scaleChanged(float);
	void scaleChanged(QString);

private:
	void setup();

	qreal m_left;
	qreal m_right;
	qreal m_top;
	qreal m_bottom;

	QPushButton	*pb_print;
	QPushButton	*pb_psetup;
	QLabel		*l_scale;
	QComboBox	*box_scale;
	QPushButton	*pb_zoomin;
	QPushButton	*pb_zoom1;
	QPushButton	*pb_zoomout;

	QTextDocument *doc;
//    QTextDocument *doct;
	QBoxLayout	*layout_main;
	QBoxLayout	*layout_butt;
	QSplitter	*split;
	PreviewView *view;
    PreviewView *thumbs;
    QPrinter printer;
};


struct PageI {
	int n;
#ifdef CACHE_PIXMAP
	QPixmap* data;
#else
	QImage* data;
#endif
};

typedef QList<PageI> PagesI;

class PreviewView : public QAbstractScrollArea
{
    Q_OBJECT
public:

enum ViewMode{
	Mode_Normal = 0,
	Mode_Thumbs = 1
};

    PreviewView(QWidget *parent, PrintPreview *printPrev, QTextDocument *document, ViewMode m = Mode_Normal);
//    PreviewView(QTextDocument *document, QWidget *printPrev);
    virtual ~PreviewView();
	void setDocument(QTextDocument *document);

    inline void updateLayout() { flushCache(); resizeEvent(0); viewport()->update(); }

	void setViewMode(ViewMode m);

/*
	void setAutoScale(bool);
	void setPageNumberVisible(bool);
	void setPageSelection(bool);
*/

	void setInterPageSpacing(int,int);
    void setPageMargin(int);
	int  offs2page(int, int, bool ck=0);
	int  page2Xoffs(int);
	int  page2Yoffs(int);
	QRect pageRect(int);

public slots:
    void zoomIn();
    void zoomOut();
    void zoom1();
    void zoomSet(float);
	void gotoPage(int);

protected:
    virtual void paintEvent(QPaintEvent *e);
    virtual void resizeEvent(QResizeEvent *);
    virtual void mousePressEvent(QMouseEvent *e);
    virtual void mouseMoveEvent(QMouseEvent *e);
    virtual void mouseReleaseEvent(QMouseEvent *e);
	PageI getPageCached(int);
	void addToCache(PageI);
	void flushCache();
	float s1,s2,s3;

private:
    void paintPage(QPainter *painter, int page);
//    void paintThumb(QImage *painter, int page);
    QTextDocument *doc;
    float scale;
    int interPageSpacingH;
    int interPageSpacingV;
    int pageMargin;
    QPoint mousePressPos;
    QPoint scrollBarValuesOnMousePress;
    PrintPreview *printPreview;

	ViewMode viewMode;
/*
	bool  autoScale;
	bool  pageNumberVisible;
	bool  pageSelection;
*/
	int   pageN;
	PagesI pages;
#ifdef MULTICOLUMN
	int		 cols;
	int		 rows;
	int		 col0offs;
#endif
	bool	 cachePages;

signals:
	void scaleChanged(float);
	void pageSelected(int);
};

#endif // PRINTPREVIEW_H
