/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2007-2009 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 *
 */

#include <device.h>

#include <tab_devinfo.h>
#include <tab_mediainfo.h>
#include <tab_transfer.h>
#include <tab_errc.h>
#include <tab_jb.h>
#include <tab_fete.h>
#include <tab_ta.h>

#include "mainwidget.h"

#include <QDebug>

QPxMainWidget::QPxMainWidget(QPxSettings *iset, devlist *idev, QWidget *p)
	: QTabWidget(p)
{
#ifndef QT_NO_DEBUG
	qDebug("STA: QPxMainWidget()");
#endif
	settings = iset;
	devices = idev;

	tab_DevInfo = new tabDevInfo(iset, idev, this);
	addTab(tab_DevInfo, tr("Device"));
	connect(this, SIGNAL(deviceSelected()), tab_DevInfo, SLOT(selectDevice()));

	tab_MediaInfo = new tabMediaInfo(iset, idev, this);
	addTab(tab_MediaInfo, tr("Media"));
	connect(this, SIGNAL(deviceSelected()), tab_MediaInfo, SLOT(selectDevice()));

	tab_RT = new tabTransfer(iset, idev, "RT", 0, this);
	addTab(tab_RT, tr("Read Transfer"));
	connect(this, SIGNAL(configured()), tab_RT, SLOT(update()));
	connect(this, SIGNAL(deviceSelected()), tab_RT, SLOT(selectDevice()));

	tab_WT = new tabTransfer(iset, idev, "WT", 1, this);
	addTab(tab_WT, tr("Write Transfer"));
	connect(this, SIGNAL(configured()), tab_WT, SLOT(update()));
	connect(this, SIGNAL(deviceSelected()), tab_WT, SLOT(selectDevice()));

	tab_ERRC = new tabERRC(iset, idev, "ERRC", this);
	addTab(tab_ERRC, tr("Error Correction"));
	connect(this, SIGNAL(configured()), tab_ERRC, SLOT(update()));
	connect(this, SIGNAL(deviceSelected()), tab_ERRC, SLOT(selectDevice()));

	tab_JB = new tabJB(iset, idev, "JB", this);
	addTab(tab_JB, tr("Jitter/Assymmethry"));
	connect(this, SIGNAL(configured()), tab_JB, SLOT(update()));
	connect(this, SIGNAL(deviceSelected()), tab_JB, SLOT(selectDevice()));

	tab_FETE = new tabFETE(iset, idev, "FETE", this);
	addTab(tab_FETE, tr("FE/TE"));
	connect(this, SIGNAL(configured()), tab_FETE, SLOT(update()));
	connect(this, SIGNAL(deviceSelected()), tab_FETE, SLOT(selectDevice()));

	tab_TA = new tabTA(iset, idev, "TA", this);
	addTab(tab_TA, tr("Time Analyser"));
	connect(this, SIGNAL(configured()), tab_TA, SLOT(update()));
	connect(this, SIGNAL(deviceSelected()), tab_TA, SLOT(selectDevice()));

#ifndef QT_NO_DEBUG
	qDebug("END: QPxMainWidget()");
#endif
}

QPxMainWidget::~QPxMainWidget()
{
#ifndef QT_NO_DEBUG
	qDebug("STA: ~QPxMainWidget()");
	qDebug("END: ~QPxMainWidget()");
#endif
}
/*
void QPxMainWidget::reconfig()
{
	emit configured();
}
*/
void QPxMainWidget::clearDev()
{
	tab_MediaInfo->clear();
	tab_DevInfo->clear();
}

void QPxMainWidget::clearMedia()
{
	tab_MediaInfo->clear();
}

void QPxMainWidget::selectDevice()
{
	emit deviceSelected();
}

void QPxMainWidget::drawGraph(QImage& img, device *dev, int ttype, int eflags)
{
	switch(ttype) {
		case TEST_RT:
			tab_RT->drawGraph(img, dev, TEST_RT);
			break;
		case TEST_WT:
			tab_WT->drawGraph(img, dev, TEST_WT);
			break;
		case TEST_ERRC:
			tab_ERRC->drawGraph(img, dev, TEST_ERRC, eflags);
			break;
		case TEST_JB:
			tab_JB->drawGraph(img, dev, TEST_JB);
			break;
		case TEST_FT:
			tab_FETE->drawGraph(img, dev, TEST_FT);
			break;
		case TEST_TA:
			break;
	}
}

