/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2008-2009 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 * */

#include <QGridLayout>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QFrame>

#include <colorlabel.h>
#include <qpxsettings.h>
#include "pref_colors.h"
#include <QDebug>

prefColors::prefColors(QPxSettings *iset, QWidget *p, Qt::WindowFlags fl)
	: QWidget(p,fl)
{
#ifndef QT_NO_DEBUG
	qDebug("STA: prefColors()");
#endif
	set = iset;
	QFrame *f;
	int idx=0;

	layout = new QGridLayout(this);
	layout->setMargin(0);
	layout->setSpacing(3);

	lc_bg = new ColorLabel(set->col_bg, tr("Background"), this);
	layout->addWidget(lc_bg,idx,0);
	lc_grid = new ColorLabel(set->col_grid, tr("Grid"), this);
	layout->addWidget(lc_grid,idx,1);
	idx++;

	f = new QFrame(this);
	f->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	layout->addWidget(f,idx,0,1,2);
	idx++;

	lc_rspeed = new ColorLabel(set->col_rspeed, tr("Read Speed"), this);
	layout->addWidget(lc_rspeed,idx,0);
	lc_wspeed = new ColorLabel(set->col_wspeed, tr("Write Speed"), this);
	layout->addWidget(lc_wspeed,idx,1);
	idx++;

	f = new QFrame(this);
	f->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	layout->addWidget(f,idx,0,1,2);
	idx++;

	lc_jitter = new ColorLabel(set->col_jitter, tr("Jitter"), this);
	layout->addWidget(lc_jitter,idx,0);
	lc_asymm = new ColorLabel(set->col_asymm, tr("asymmmethry"), this);
	layout->addWidget(lc_asymm,idx,1);
	idx++;

	f = new QFrame(this);
	f->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	layout->addWidget(f,idx,0,1,2);
	idx++;

	lc_errc[0] = new ColorLabel(*set->col_errc.raw[0], tr("BLER"), this);
	layout->addWidget(lc_errc[0],idx,0);
	lc_errc[4] = new ColorLabel(*set->col_errc.raw[4], tr("E12/POE"), this);
	layout->addWidget(lc_errc[4],idx,1);
	idx++;
	lc_errc[1] = new ColorLabel(*set->col_errc.raw[1], tr("E11/PIE"), this);
	layout->addWidget(lc_errc[1],idx,0);
	lc_errc[5] = new ColorLabel(*set->col_errc.raw[5], tr("E22/POsum8"), this);
	layout->addWidget(lc_errc[5],idx,1);
	idx++;
	lc_errc[2] = new ColorLabel(*set->col_errc.raw[2], tr("E21/PIsum8"), this);
	layout->addWidget(lc_errc[2],idx,0);
	lc_errc[6] = new ColorLabel(*set->col_errc.raw[6], tr("E32/POF"), this);
	layout->addWidget(lc_errc[6],idx,1);
	idx++;
	lc_errc[3] = new ColorLabel(*set->col_errc.raw[3], tr("E31/PIF"), this);
	layout->addWidget(lc_errc[3],idx,0);
	lc_errc[7] = new ColorLabel(*set->col_errc.raw[7], tr("UNCR"), this);
	layout->addWidget(lc_errc[7],idx,1);
	idx++;

	f = new QFrame(this);
	f->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	layout->addWidget(f,idx,0,1,2);
	idx++;

	lc_fe = new ColorLabel(set->col_fe, tr("Focus Errors"), this);
	layout->addWidget(lc_fe,idx,0);
	lc_te = new ColorLabel(set->col_te,tr("Tracking errors"), this);
	layout->addWidget(lc_te,idx,1);
	idx++;

	f = new QFrame(this);
	f->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	layout->addWidget(f,idx,0,1,2);
	idx++;

	lc_tapit = new ColorLabel(set->col_tapit, tr("TA pits"), this);
	layout->addWidget(lc_tapit,idx,0);
	lc_taland = new ColorLabel(set->col_taland,tr("TA lands"), this);
	layout->addWidget(lc_taland,idx,1);
	idx++;

	layout->setRowStretch(idx, 10);

#ifndef QT_NO_DEBUG
	qDebug("END: prefColors()");
#endif
}

prefColors::~prefColors()
{
#ifndef QT_NO_DEBUG
	qDebug("STA: ~prefColors()");
#endif
	set->col_bg     = lc_bg->color();
	set->col_bginv  = QColor( (~set->col_bg.red()) & 0xFF, (~set->col_bg.green()) & 0xFF, (~set->col_bg.blue()) & 0xFF );
	set->col_grid   = lc_grid->color();

	set->col_rspeed = lc_rspeed->color();
	set->col_wspeed = lc_wspeed->color();

	for (int i=0;i<8;i++)
		*set->col_errc.raw[i] = lc_errc[i]->color();

	set->col_jitter = lc_jitter->color();
	set->col_asymm  = lc_asymm->color();
	set->col_fe		= lc_fe->color();
	set->col_te		= lc_te->color();
	set->col_tapit  = lc_tapit->color();
	set->col_taland = lc_taland->color();
#ifndef QT_NO_DEBUG
	qDebug("END: ~prefColors()");
#endif
}

