/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2008-2009 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * this file is based on textedit.cpp (TextEdit demo) from Qt 4.3.2
 * Copyright (C) 2004-2008 Trolltech ASA. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 * */

#include "printpreview.h"

#include <QPrintDialog>
#include <QPrinter>
#include <QToolBar>
#include <QAction>
#include <QTextFormat>
#include <QMouseEvent>
#include <QTextFrame>
#include <QTextDocument>
#include <QAbstractTextDocumentLayout>
#include <QScrollBar>
#include <QPainter>
#include <QDebug>
#include <QPageSetupDialog>
#include <QSplitter>

#include <QLabel>
#include <QComboBox>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>

/*
#include <images/printer.xpm>
#include <images/zoom_in.xpm>
#include <images/zoom_out.xpm>
#include <images/zoom_orig.xpm>
*/

PreviewView::PreviewView(QWidget *parent, PrintPreview *printPrev, QTextDocument *document, ViewMode m)
//PreviewView::PreviewView(QTextDocument *document, QWidget *printPrev)
	: QAbstractScrollArea(parent), printPreview(printPrev)
{
#ifndef QT_NO_DEBUG
	qDebug("STA: PreviewView()");
#endif
    verticalScrollBar()->setSingleStep(20);
    horizontalScrollBar()->setSingleStep(20);

    viewport()->setBackgroundRole(QPalette::Dark);

    doc = document;
	if (doc) {
		QFont f(doc->defaultFont());
		f.setPointSize(DEFAULT_FONT_SIZE);
		doc->setDefaultFont(f);
	}
	viewMode = m;
/*
	autoScale = 0;
	pageNumberVisible = 0;
	pageSelection = 0;
*/
#ifdef MULTICOLUMN
	cols = 1;
#endif
	pageN = 0;
	if (viewMode == Mode_Thumbs) {
		scale = THUMBS_DEF_SCALE;
		cachePages = 1;
#ifdef SET_BACKGROUND
		viewport()->setBackgroundRole(QPalette::Light);
#endif
	} else {
		scale = 1.0;
		cachePages = 0;
#ifdef SET_BACKGROUND
		viewport()->setBackgroundRole(QPalette::Dark);
#endif
	}
/*
#if ((MULTICOLUMN == 1) && (CACHE_ALL == 1))
	cachePages = 1;
#endif
*/
	interPageSpacingH = 10;
	interPageSpacingV = 10;
    pageMargin = 5;
	flushCache();
#ifndef QT_NO_DEBUG
	qDebug("END: PreviewView()");
#endif
}

PreviewView::~PreviewView()
{
#ifndef QT_NO_DEBUG
	qDebug("     ~PreviewView()");
#endif
/*

#ifndef QT_NO_DEBUG
	wDebug("END: ~PreviewView()");
#endif
*/
}

void PreviewView::setDocument(QTextDocument *document)
{
	flushCache();
	doc = document;

	if (doc) {
		QFont f(doc->defaultFont());
		f.setPointSize(DEFAULT_FONT_SIZE);
		doc->setDefaultFont(f);
	}
}

/*
void PreviewView::setAutoScale(bool as)
{
	if (autoScale == as) return;
	autoScale = as;
	if (!as) return;
	scale = (float)(width() - 20 - 2 * pageMargin) / (float)printPreview->paperSize.width();
    viewport()->update();
}

void PreviewView::setPageNumberVisible(bool s)
{
	pageNumberVisible = s;
    viewport()->update();
}

void PreviewView::setPageSelection(bool s)
{
	pageSelection = s;
}
*/

void PreviewView::setInterPageSpacing(int h, int v)
{
    interPageSpacingH = h;
    interPageSpacingV = v;
    viewport()->update();
}

void PreviewView::setPageMargin(int i)
{
    pageMargin = i;
//    setMinimumWidth(i*4 + 40);
    viewport()->update();
}

void PreviewView::setViewMode(ViewMode m)
{
	viewMode = m;
	if (viewMode == Mode_Thumbs) {
		scale = THUMBS_DEF_SCALE;
#ifdef SET_BACKGROUND
		viewport()->setBackgroundRole(QPalette::Light);
#endif
	} else {
		scale = 1.0;
#ifdef SET_BACKGROUND
		viewport()->setBackgroundRole(QPalette::Dark);
#endif
	}
}

void PreviewView::zoomIn()
{	
	if (scale>=4.00 || viewMode == Mode_Thumbs) return;
#ifdef ZOOM_STEP_MULT
	zoomSet(scale *= SIZE_STEP);
#else
	zoomSet(scale += 0.01);
#endif
	emit scaleChanged(scale);
 }

void PreviewView::zoomOut()
{
	if (scale<=0.25 || viewMode == Mode_Thumbs) return;
#ifdef ZOOM_STEP_MULT
	zoomSet(scale /= SIZE_STEP);
#else
	zoomSet(scale -= 0.01);
#endif
	emit scaleChanged(scale);
}

void PreviewView::zoom1()
{
	if (viewMode == Mode_Thumbs) return; 
	zoomSet(1.0);
   	emit scaleChanged(scale);
}

void PreviewView::zoomSet(float s)
{
	if (viewMode == Mode_Thumbs) return; 
	flushCache();
	if (s<0.25) scale = 0.25;
	else if (s>4.00) scale = 4.00;
	else scale = s;
	s1 = 1/scale;
	s2 = 2/scale;
	s3 = 3/scale;
    resizeEvent(0);
    viewport()->update();
//	emit scaleChanged(scale);
}

/*
int PreviewView::Xoffs2page(int offs)
{
	return 0;
//	return ( offs - pageMargin ) / (printPreview->paperSize.height() * scale + interPageSpacing);
}
*/
int PreviewView::offs2page(int offsX, int offsY, bool ck)
{
	QRect r;
	int pn=0,pn2=0;
#ifdef MULTICOLUMN
	if (!cols)
#endif
		pn = (int)((double)( offsY - pageMargin ) / (printPreview->paperSize.height() * scale + interPageSpacingV));
#ifdef MULTICOLUMN
	pn = cols * (int)((double)( offsY - pageMargin) / (printPreview->paperSize.height() * scale + interPageSpacingV)) + 
		(int)((double)( offsX - col0offs + (interPageSpacingH >> 1) ) / (printPreview->paperSize.width() * scale + interPageSpacingH));
		//(int)((double)( offsX - pageMargin + (interPageSpacingH >> 1) ) / (printPreview->paperSize.width() * scale + interPageSpacingH));
#endif
	if (pn>= doc->pageCount())pn=-1;
	r = pageRect(pn);
	pn2 = r.contains(offsX, offsY) ? pn : -1;
#if 0
	wDebug(QString("%1.%2 : page %3. pageRect: %4.%5 - %6.%7, npage: %8")
			.arg(offsX)
			.arg(offsY)
			.arg(pn)
			.arg(r.x())
			.arg(r.y())
			.arg(r.x() + r.width()-1)
			.arg(r.y() + r.height()-1)
			.arg(pn2));
#endif
	if (!ck) {
		return pn;
	} else {
		return pn2;
	}
}

int PreviewView::page2Xoffs(int pn)
{
	int col=1;
#ifdef MULTICOLUMN
	col = pn % cols;
#endif
	return col0offs + (int)((printPreview->paperSize.width()*scale + interPageSpacingH) * col);
}

int PreviewView::page2Yoffs(int pn)
{
#ifdef MULTICOLUMN
	int row;
	if (cols<2)
#endif
		return (int)((printPreview->paperSize.height() * scale + interPageSpacingV) * pn + pageMargin);
#ifdef MULTICOLUMN
	row = pn / cols;
	return (int)((printPreview->paperSize.height() * scale + interPageSpacingV) * row + pageMargin);
#endif
}

QRect PreviewView::pageRect(int pn)
{
	return QRect(page2Xoffs(pn),
				page2Yoffs(pn),
				printPreview->paperSize.width()*scale,
				printPreview->paperSize.height()*scale);
}

void PreviewView::gotoPage(int pn)
{
	verticalScrollBar()->setValue(page2Yoffs(pn) - pageMargin);
}

void PreviewView::paintEvent(QPaintEvent *)
{
#ifdef PP_DEBUG2
	wDebug("STA: PreviewView::paintEvent()");
#endif
#ifdef MULTICOLUMN
	int col=0;
#endif
	QPainter *p;
	QColor   bgc;
//	QImage *pimg;
	int page0, page1;
    int pages;
	PageI pagei;

	if (!doc) return;
	
	pages = doc->pageCount();

	page0 = offs2page (pageMargin, verticalScrollBar()->value());
#ifdef MULTICOLUMN
	page1 = offs2page (viewport()->width(), verticalScrollBar()->value() + viewport()->height());
#else
	page1 = offs2page (pageMargin, verticalScrollBar()->value() + viewport()->height());
#endif
	if (page0 < 0) page0 = 0;
	if (page1 > pages-1 || page1 <0) page1 = pages-1;
#ifdef PP_DEBUG2
		wDebug(QString("pages: %1 .. %2 (of %3)").arg(page0).arg(page1).arg(pages));
#endif

	p = new QPainter(viewport());
	p->translate(-horizontalScrollBar()->value()-1, -verticalScrollBar()->value()-1);
	if (viewport()->width() > (printPreview->paperSize.width() * scale + 2 * pageMargin)) {
#ifdef MULTICOLUMN
//		qWarning("rows: %d, cols: %d",rows,cols);
		p->translate(page2Xoffs(page0),
//		p->translate((viewport()->width() - printPreview->paperSize.width()*scale*cols + interPageSpacing*(cols-1)) / 2,
			page2Yoffs(page0));
#else
		p->translate(page2Xoffs(page0),
//		p->translate((viewport()->width() - printPreview->paperSize.width()*scale) / 2,
			page2Yoffs(page0));
#endif
	} else {
		p->translate(pageMargin, page2Yoffs(page0));
	}
#ifdef MULTICOLUMN
	col = 0;
#endif
//		for (int i = 0; i < pages; ++i) {
	for (int i = page0; i <= page1; ++i) {
		if (cachePages) {
//		if (viewMode == Mode_Thumbs || cols > 1) {
#ifdef PP_DEBUG2
			wDebug("paintEvent: Cache");
#endif
			pagei = getPageCached(i);
/*
			QImage pimg2((int)(printPreview->paperSize.width() * scale + pageMargin*2),
						(int)(printPreview->paperSize.height() * scale + interPageSpacing),
						QImage::Format_ARGB32);
//			pimg2.fill(Qt::white);
			QPainter *p3 = new QPainter(&pimg2);
			paintPage(p3, i);
			delete p3;
*/
			if (!pagei.data) {
				bgc = palette().color((viewport()->backgroundRole()));
/*
				if (viewMode == Mode_Thumbs)
					bgc = palette().light().color();
				else
					bgc = palette().dark().color();
*/
				pagei.n = i;
#ifdef CACHE_PIXMAP
				pagei.data = new QPixmap(
						(int)(printPreview->paperSize.width() * scale + pageMargin*2),
						(int)(printPreview->paperSize.height() * scale + interPageSpacingV));
				pix.fill(bgc);
#else

#ifdef DRAW_THRU_PIX
				QPixmap pix((int)(printPreview->paperSize.width() * scale + pageMargin*2),
						(int)(printPreview->paperSize.height() * scale + interPageSpacingV));
				pix.fill(bgc);
#else
				pagei.data = new QImage(
						(int)(printPreview->paperSize.width() * scale + pageMargin*2),
						(int)(printPreview->paperSize.height() * scale + interPageSpacingV),
						//QImage::Format_ARGB32_Premultiplied
						QImage::Format_ARGB32
						//QImage::Format_RGB32
						);
				pagei.data->fill(bgc);
#endif

#endif

#if (!defined CACHE_PIXMAP && defined DRAW_THRU_PIX)
				QPainter *p2 = new QPainter(&pix);
#else
				QPainter *p2 = new QPainter(pagei.data);
#endif
				paintPage(p2, i);
				delete p2;
#if (!defined CACHE_PIXMAP && defined DRAW_THRU_PIX)
				pagei.data = new QImage( pix.toImage() );
#endif
			//	if (viewMode == Mode_Normal)
			//		pimg->save(QString("page-%1_scale-%2_x.png").arg(i).arg(scale, 3, 'f'),"PNG");
				addToCache(pagei);
			}
#ifdef PP_DEBUG2
			wDebug(QString("pagei.data @0x%1").arg((int)pagei.data,8,16,QChar('0')));
#endif
#ifdef CACHE_PIXMAP
			p->drawPixmap(0,0,*(pagei.data));
#else
			p->drawImage(0,0,*(pagei.data));
#endif
			//			p->drawImage(0,0,pimg2);
		} else {
#ifdef PP_DEBUG2
			wDebug("paintEvent: Direct");
#endif
			p->save();
			paintPage(p, i);
			p->restore();
		}
		// paint selection rect
		if (viewMode == Mode_Thumbs && i == pageN) {
			p->setCompositionMode(QPainter::CompositionMode_Multiply);	
			//QColor col = QColor(0x70, 0x70, 0xA0, 0x80);
			QColor col = palette().color(QPalette::Highlight);
			col.setAlpha(0x80);
			p->setPen(col);
			p->setBrush(col);
			p->drawRect(QRectF(
					0, 0,
					printPreview->paperSize.width() * scale,
					printPreview->paperSize.height() * scale));
			p->setCompositionMode(QPainter::CompositionMode_SourceOver);
		}
#ifdef MULTICOLUMN
		col++;
		if (col<cols) {
			p->translate(interPageSpacingH + printPreview->paperSize.width() * scale, 0);
		} else {
			col = 0;
			p->translate( -(interPageSpacingH + printPreview->paperSize.width() * scale)*(cols-1),
				interPageSpacingV + printPreview->paperSize.height() * scale);
		}
#else
		p->translate(0, interPageSpacingV + printPreview->paperSize.height() * scale);
#endif

	}
	delete p;
#ifdef PP_DEBUG2
	wDebug("END: PreviewView::paintEvent()");
#endif
}

void PreviewView::paintPage(QPainter *painter, int page)
{
#ifdef PP_DEBUG2
	wDebug(QString("STA: PreviewView::paintPage(%1)").arg(page));
#endif
    const QSizeF pgSize = doc->pageSize();
    QColor col, bgc;
	float w = printPreview->paperSize.width();
	float h = printPreview->paperSize.height();
	QFont of,f;
//	unsigned char bgR, bgG, bgB;

#ifdef PP_DEBUG2
//	wDebug(QString("scale: %1").arg(scale, 3, 'f'));
#endif
    painter->scale(scale, scale);

	painter->save();

// background
	col = Qt::black;
    painter->setPen(col);
    painter->setBrush(Qt::white);
	painter->drawRect(QRectF(QPointF(0, 0), printPreview->paperSize));
    painter->setBrush(Qt::NoBrush);

    QRectF docRect(QPointF(0, page * pgSize.height()), pgSize);
    QAbstractTextDocumentLayout::PaintContext ctx;
    ctx.clip = docRect;

	//if (pageNumberVisible) {
	if (viewMode == Mode_Thumbs) {
		of = painter->font();
		f = of;
		f.setPointSizeF(1.2*of.pointSizeF()/scale);
		//f.setPointSizeF(36);
		painter->setFont(f);
#ifdef PP_DEBUG3
		wDebug("showPageNumber");
#endif
		col = Qt::black;
		painter->setPen(col);
		painter->drawText(0,h+s2,w,h+s1*interPageSpacingH, Qt::AlignTop | Qt::AlignHCenter, QString::number(page+1));
		painter->setFont(of);
	}
    // don't use the system palette text as default text color, on HP/UX
    // for example that's white, and white text on white paper doesn't
    // look that nice
    ctx.palette.setColor(QPalette::Text, Qt::black);

    painter->translate(0, - page * pgSize.height());
//    painter->translate(printPreview->pageTopLeft);
    painter->setClipRect(docRect);//.translated(printPreview->pageTopLeft));
#ifdef PP_DEBUG2
//	wDebug(QString("font size=%1").arg(painter->font().pointSizeF()));
#endif
	//painter->setRenderHint(QPainter::Antialiasing, true);
	painter->setRenderHint(QPainter::TextAntialiasing, true);
	doc->documentLayout()->draw(painter, ctx);

	painter->restore();

    painter->setPen(Qt::white);
    painter->setBrush(Qt::white);
    painter->drawRect(QRectF(QPointF(0, 0),
				QSize( w, inchesToPixelsX(mmToInches(DEFAULT_MARGIN_TOP), this) )));
    painter->drawRect(QRectF(QPointF(0, h - inchesToPixelsX(mmToInches(DEFAULT_MARGIN_BOTTOM), this)),
				QSize( w, inchesToPixelsX(mmToInches(DEFAULT_MARGIN_BOTTOM), this) )));

// border
	col = Qt::black;
    painter->setPen(col);
    painter->setBrush(Qt::NoBrush);
	painter->drawRect(QRectF(QPointF(0, 0), printPreview->paperSize));

	bgc = palette().color((viewport()->backgroundRole()));
    col = QColor(bgc.red()*0.4, bgc.green()*0.4, bgc.blue()*0.4);
    painter->setPen(col);
    painter->drawLine(QLineF(w+s1, s3,   w+s1, h+s1));
	painter->drawLine(QLineF(s3,   h+s1, w+s1, h+s1));

    col = QColor(bgc.red()*0.6, bgc.green()*0.6, bgc.blue()*0.6);
    painter->setPen(col);
    painter->drawLine(QLineF(w+s2, s3,   w+s2, h+s2));
	painter->drawLine(QLineF(s3,   h+s2, w+s2, h+s2));

    col = QColor(bgc.red()*0.8, bgc.green()*0.8, bgc.blue()*0.8);
    painter->setPen(col);
    painter->drawLine(QLineF(w+s3, s3,   w+s3, h+s3));
	painter->drawLine(QLineF(s3,   h+s3, w+s3, h+s3));

#ifdef PP_DEBUG2
	wDebug("END: PreviewView::paintPage()");
#endif
}

PageI PreviewView::getPageCached(int page)
{
	PageI pagei;
	pagei.n=-1;
	pagei.data=NULL;
#ifdef PP_DEBUG2
	wDebug(QString("PreviewView::getPageCached(%1), cached: %2").arg(page).arg(pages.size()));
#endif
	for (int i=0; i<pages.size(); i++) {
		if (pages[i].n == page) {
			pagei = pages.takeAt(i);
			pages.prepend(pagei);
			return pagei;
		}
	}
	return pagei;
}

void PreviewView::addToCache(PageI pagei)
{
#ifdef PP_DEBUG2
	wDebug(QString("PreviewView::addToCache(%1,%2)").arg(pagei.n).arg((int)pagei.data,8,16,QChar('0')));
#endif
	pages.prepend(pagei);
	if (pages.size()>PAGES_CACHED_MAX) {
		delete pages.takeLast().data;
	}
}

void PreviewView::flushCache()
{
#ifdef PP_DEBUG2
	wDebug("PreviewView::flushCache()");
#endif
	for (int i=pages.size(); i>0; i--)
		delete pages.takeLast().data;
	pages.clear();
}

void PreviewView::resizeEvent(QResizeEvent *)
{
#ifdef PP_DEBUG2
	wDebug(QString("STA: PreviewView::resizeEvent() doc@ 0x%1 as = %2").arg((int)doc,8,16,QChar('0')).arg((int)viewMode));
#endif
	if (!doc) {
		horizontalScrollBar()->setRange(0, 0);
		verticalScrollBar()->setRange(0, 0);
		return;
	}
	const int pageCount = doc->pageCount();

	//if (autoScale)
#ifdef THUMBS_AUTOSCALE
	if (viewMode == Mode_Thumbs) {
		flushCache();
		cachePages = 1;
		scale = (float)(width() - 20 - 2 * pageMargin) / (float)printPreview->paperSize.width();
		s1 = 1/scale;
		s2 = 2/scale;
		s3 = 3/scale;
	}
#endif
#ifdef MULTICOLUMN
#ifndef THUMBS_MULTICOLUMN
	if (viewMode == Mode_Thumbs) {
		cols = 1;
		rows = pageCount;
	} else {
#endif
		cols = (int) ((viewport()->width() - 2 * pageMargin + interPageSpacingH) / (printPreview->paperSize.width() * scale + interPageSpacingH));
		if (!cols) cols = 1;
		if (cols > pageCount) cols = pageCount;
#ifndef THUMBS_MULTICOLUMN
	}
#endif
	rows = (pageCount+cols-1) / cols;
	if (viewport()->width() > (printPreview->paperSize.width()*scale - 2*pageMargin))
		col0offs = (int) (viewport()->width() - printPreview->paperSize.width()*scale*cols - (interPageSpacingH*(cols-1))) >> 1;
	else
		col0offs = pageMargin;
#endif

#ifdef CACHE_ALWAYS_EN
	cachePages = 1;
#else

#if (defined (MULTICOLUMN ) && defined(CACHE_ALL))
	if (viewMode == Mode_Thumbs || (viewMode == Mode_Normal && cols>1)) {
#else
	if (viewMode == Mode_Thumbs) {
#endif
		cachePages = 1;
	} else {
		cachePages = 0;
	}
#endif // CACHE_ALWAYS_EN


#if (!defined(MULTICOLUMN) || !defined(CACHE_ALL))
	if (viewMode == Mode_Thumbs) {
		cachePages = 1;
	} else {
		cachePages = 0;
	}
#endif

	const QSize viewportSize = viewport()->size();
	QSize docSize;
    docSize.setWidth(qRound(printPreview->paperSize.width() * scale + 2 * pageMargin));
#ifdef MULTICOLUMN
	if (viewMode == Mode_Thumbs)
		docSize.setHeight(qRound(rows* printPreview->paperSize.height() * scale + rows * interPageSpacingV + pageMargin));
	else
		docSize.setHeight(qRound(rows* printPreview->paperSize.height() * scale + (rows-1) * interPageSpacingV + 2 * pageMargin));
#else
	if (viewMode == Mode_Thumbs)
		docSize.setHeight(qRound(pageCount * printPreview->paperSize.height() * scale + (pageCount) * interPageSpacingV + pageMargin));
	else
		docSize.setHeight(qRound(pageCount * printPreview->paperSize.height() * scale + (pageCount-1) * interPageSpacingV + 2 * pageMargin));
#endif
    horizontalScrollBar()->setRange(0, docSize.width() - viewportSize.width());
    horizontalScrollBar()->setPageStep(viewportSize.width());

    verticalScrollBar()->setRange(0, docSize.height() - viewportSize.height());
    verticalScrollBar()->setPageStep(viewportSize.height());
#ifdef PP_DEBUG2
	wDebug("END: PreviewView::resizeEvent()");
#endif
}

void PreviewView::mousePressEvent(QMouseEvent *e)
{
	int pn;
	int ox,oy;
#ifdef PP_DEBUG3
	wDebug(QString("mouse: %1 %2").arg(e->x()).arg(e->y()));
#endif
	//if (pageSelection) {
//	if (viewMode == Mode_Thumbs) {
#ifndef QT_NO_DEBUG 
	//wDebug(QString("%1.%2 : page %3. page2offs: %4.%5").arg(ox).arg(oy).arg(pn).arg(page2Xoffs(pn)).arg(page2Yoffs(pn)));
#endif 

	if (viewMode == Mode_Thumbs) {
		pn = offs2page (horizontalScrollBar()->value() + e->x(), verticalScrollBar()->value() + e->y(),1);
		if (pn>=0 && pn < doc->pageCount()) {
			pageN = pn;
			viewport()->update();
			emit pageSelected(pageN);
		}
	}
	mousePressPos = e->pos();
    scrollBarValuesOnMousePress.rx() = horizontalScrollBar()->value();
    scrollBarValuesOnMousePress.ry() = verticalScrollBar()->value();
    e->accept();
}

void PreviewView::mouseMoveEvent(QMouseEvent *e)
{
    if (mousePressPos.isNull()) {
        e->ignore();
        return;
    }

    horizontalScrollBar()->setValue(scrollBarValuesOnMousePress.x() - e->pos().x() + mousePressPos.x());
    verticalScrollBar()->setValue(scrollBarValuesOnMousePress.y() - e->pos().y() + mousePressPos.y());
    horizontalScrollBar()->update();
    verticalScrollBar()->update();
    e->accept();
}

void PreviewView::mouseReleaseEvent(QMouseEvent *e)
{
    mousePressPos = QPoint();
    e->accept();
}

PrintPreview::PrintPreview(QWidget *parent, const QTextDocument *document)
//    : QMainWindow(parent), printer(QPrinter::HighResolution)
    : QDialog(parent), printer(QPrinter::HighResolution)
{
#ifndef QT_NO_DEBUG
	qDebug("STA: PrintPreview()");
#endif
	QList<int> ss;
	ss.append(100);
	ss.append(650);

	setWindowFlags(Qt::Window);
    setWindowTitle(tr("Print Preview"));

    printer.setFullPage(true);
	if (document) {
		doc = document->clone();
	} else {
		doc = new QTextDocument;
	}
//    doct = doc->clone();

	layout_main = new QVBoxLayout(this);
	layout_main->setMargin(3);
	layout_main->setSpacing(3);
	setLayout(layout_main);

	layout_butt = new QHBoxLayout;
	layout_butt->setMargin(0);
	layout_butt->setSpacing(3);
	layout_main->addLayout(layout_butt);

	split = new QSplitter(this);
	layout_main->addWidget(split);
/*
	layout_view = new QHBoxLayout;
	layout_view->setMargin(0);
	layout_view->setSpacing(3);
	layout_main->addLayout(layout_view);
*/
    thumbs = new PreviewView(split, this, doc, PreviewView::Mode_Thumbs);
//    layout_view->addWidget(thumbs);
//	thumbs->zoomSet(0.01);

#ifdef THUMBS_MULTICOLUMN
	thumbs->setMinimumWidth( 100 );
	thumbs->setMaximumWidth( 300 );
#else
	thumbs->setMinimumWidth( 120 );
	thumbs->setMaximumWidth( 120 );
#endif
	thumbs->setInterPageSpacing(5,20);
	thumbs->setPageMargin(5);
/*
	thumbs->setAutoScale(true);
	thumbs->setPageNumberVisible(true);
	thumbs->setPageSelection(true);
*/
	thumbs->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);

    view = new PreviewView(split, this, doc);
//    layout_view->addWidget(view);

    doc->documentLayout()->setPaintDevice(view->viewport());
//    doct->documentLayout()->setPaintDevice(thumbs->viewport());

	setDefaults(doc, &printer, this);
	setup();

//    QToolBar *tb = addToolBar(tr("Print"));
//    tb->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);

	pb_print = new QPushButton(this);
	pb_print->setText(tr("Print"));
	pb_print->setIcon(QIcon(":images/printer.png"));
	layout_butt->addWidget(pb_print);

	pb_psetup = new QPushButton(this);
	pb_psetup->setText(tr("Page Setup"));
	layout_butt->addWidget(pb_psetup);

	layout_butt->addStretch(10);

	l_scale = new QLabel(tr("Scale"),this);
	layout_butt->addWidget(l_scale);

	box_scale = new QComboBox(this);
	box_scale->setMaximumHeight(24);
	box_scale->setMinimumHeight(24);
	box_scale->addItem(tr("25%"));
	box_scale->addItem(tr("50%"));
	box_scale->addItem(tr("100%"));
	box_scale->addItem(tr("200%"));
	box_scale->addItem(tr("400%"));
	box_scale->setEditable(true);
	layout_butt->addWidget(box_scale);

	pb_zoomin = new QPushButton(this);
//	pb_zoomin->setText(tr("Zoom In"));
	pb_zoomin->setToolTip(tr("Zoom In"));
	pb_zoomin->setIcon(QIcon(":images/zoom-in.png"));
	layout_butt->addWidget(pb_zoomin);

	pb_zoomout = new QPushButton(this);
//	pb_zoomout->setText(tr("Zoom Out"));
	pb_zoomout->setToolTip(tr("Zoom Out"));
	pb_zoomout->setIcon(QIcon(":images/zoom-out.png"));
	layout_butt->addWidget(pb_zoomout);

	pb_zoom1 = new QPushButton(this);
//	pb_zoom1->setText(tr("Zoom 100%"));
	pb_zoom1->setToolTip(tr("Zoom 100%"));
	pb_zoom1->setIcon(QIcon(":images/zoom-orig.png"));
	layout_butt->addWidget(pb_zoom1);

	resize(750, 650);
	split->setSizes(ss);

	connect(thumbs, SIGNAL(pageSelected(int)), view, SLOT(gotoPage(int)));
	connect(view, SIGNAL(scaleChanged(float)), this, SLOT(scaleChanged(float)));
	connect(box_scale, SIGNAL(currentIndexChanged(QString)), this, SLOT(scaleChanged(QString)));
    connect(pb_print, SIGNAL(clicked()), this, SLOT(print()));
    connect(pb_psetup, SIGNAL(clicked()), this, SLOT(pageSetup()));
	connect(pb_zoomin, SIGNAL(clicked()), view, SLOT(zoomIn()));
    connect(pb_zoomout, SIGNAL(clicked()), view, SLOT(zoomOut()));
    connect(pb_zoom1, SIGNAL(clicked()), view, SLOT(zoom1()));

	view->zoom1();

#ifndef QT_NO_DEBUG
	qDebug("END: PrintPreview()");
#endif
}

PrintPreview::~PrintPreview()
{
#ifndef QT_NO_DEBUG
	qDebug("STA: ~PrintPreview()");
#endif
    delete doc;
//    delete doct;
#ifndef QT_NO_DEBUG
	qDebug("END: ~PrintPreview()");
#endif
}

//void PrintPreview::setDefaults(QTextDocument *doc, QPrinter *printer, QWidget *view)
void PrintPreview::setDefaults(QTextDocument *doc, QPrinter *printer, QWidget *view)
{
	doc->setUseDesignMetrics(true);
    QFont f(doc->defaultFont());
    f.setPointSize(10);
    doc->setDefaultFont(f);

	// set margins
    QTextFrameFormat fmt = doc->rootFrame()->frameFormat();
	m_left   = DEFAULT_MARGIN_LEFT;
	m_right  = DEFAULT_MARGIN_RIGHT;
	m_top    = DEFAULT_MARGIN_TOP;
	m_bottom = DEFAULT_MARGIN_BOTTOM;
#if QT_VERSION >= 0x040400
	printer->setPageMargins(m_left, m_top, m_right, m_bottom, QPrinter::Millimeter);
#endif
	fmt.setLeftMargin(inchesToPixelsX(mmToInches(m_left), view));
    fmt.setRightMargin(inchesToPixelsX(mmToInches(m_right), view));
    fmt.setTopMargin(inchesToPixelsY(mmToInches(m_top), view));
    fmt.setBottomMargin(inchesToPixelsY(mmToInches(m_bottom), view));
	doc->rootFrame()->setFrameFormat(fmt);

//	printer->setPageRect("");
}

void PrintPreview::setup()
{
//	setup(doc, &printer, this);
//	setup(doct, &printer, this);

//	doct->setUseDesignMetrics(true);
//	doct->setDefaultFont(doc->defaultFont());
//	doct->rootFrame()->setFrameFormat(doc->rootFrame()->frameFormat());

	QSizeF page = printer.pageRect().size();
    paperSize = printer.paperRect().size();
//	wDebug(QString("paperSize: %1 x %2").arg(paperSize.width()).arg(paperSize.height()));
    paperSize.rwidth() *= float(view->logicalDpiX()) / printer.logicalDpiX();
    paperSize.rheight() *= float(view->logicalDpiY()) / printer.logicalDpiY();

	QPointF pageTopLeft = printer.pageRect().topLeft();
//	wDebug(QString("pageTopLeft: %1, %2").arg(pageTopLeft.rx()).arg(pageTopLeft.ry()));
	pageTopLeft.rx() *= float(view->logicalDpiX()) / printer.logicalDpiX();
	pageTopLeft.ry() *= float(view->logicalDpiY()) / printer.logicalDpiY();

//	wDebug(QString("pageSize: %1 x %2").arg(page.width()).arg(page.height()));
    page.setWidth(page.width() * view->logicalDpiX() / printer.logicalDpiX());
    page.setHeight(page.height() * view->logicalDpiY() / printer.logicalDpiY());
    doc->setPageSize(page);

#if QT_VERSION >= 0x040400
	printer.getPageMargins(&m_left, &m_top, &m_right, &m_bottom, QPrinter::Millimeter);
#endif
	QTextFrameFormat fmt = doc->rootFrame()->frameFormat();
	fmt.setLeftMargin(inchesToPixelsX(mmToInches(m_left), view));
    fmt.setRightMargin(inchesToPixelsX(mmToInches(m_right), view));
    fmt.setTopMargin(inchesToPixelsY(mmToInches(m_top), view));
    fmt.setBottomMargin(inchesToPixelsY(mmToInches(m_bottom), view));
	doc->rootFrame()->setFrameFormat(fmt);
}

void PrintPreview::setDocument(const QTextDocument *document)
{
#ifdef PP_DEBUG2
	wDebug(QString("STA: PreviewView::setDocument() document@ 0x%1").arg((int)document,8,16,QChar('0')));
#endif
	if (!document) return;
	view->setDocument(NULL);
	thumbs->setDocument(NULL);
	delete doc; doc = 0;
	doc = document->clone();
    doc->documentLayout()->setPaintDevice(view->viewport());
	view->setDocument(doc);
	thumbs->setDocument(doc);
	setDefaults(doc, &printer, this);
	setup();
}

void PrintPreview::print()
{
    QPrintDialog *dlg = new QPrintDialog(&printer, this);
    if (dlg->exec() == QDialog::Accepted) {
        doc->print(&printer);
    }
    delete dlg;
}

void PrintPreview::pageSetup()
{
    QPageSetupDialog dlg(&printer, this);
    if (dlg.exec() == QDialog::Accepted) {
		setup();
        view->updateLayout();
        thumbs->updateLayout();
    }
}

void PrintPreview::scaleChanged(float scale)
{
	int idx = box_scale->findText(QString("%1%").arg((int)(scale*100)));
	if (idx<0) {
		//box_scale->setEditText(QString("%1%").arg(scale*100,2,'f'));
		box_scale->setEditText(QString("%1%").arg((int)(scale*100)));
	} else {
		box_scale->setCurrentIndex(idx);
	}
}

void PrintPreview::scaleChanged(QString scale)
{
#ifndef QT_NO_DEBUG
	qDebug("scaleChanged()");
#endif
	scale.remove('%');
	view->zoomSet(scale.toInt()/100.0);
}

