/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2008-2009 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 *
 */

#include <QSettings>
#include "qpxsettings.h"

#include <QDebug>

#define DEF_COL_BG		0xFFFFFF
#define DEF_COL_GRID	0x808080

#define DEF_COL_RSPEED	0x009000
#define DEF_COL_WSPEED	0xC00000

const unsigned int DEF_COL_ERRC[] = {
	0x009000,
	0x00C000,
	0x00FF00,
	0xFFAA00,
	0x0055FF,
	0x550080,
	0xFF0000,
	0x000000
};

#define DEF_COL_JITTER	0x00A000
#define DEF_COL_ASYMM	0xA00000
#define DEF_COL_FE		0xA00000
#define DEF_COL_TE		0x0000A0
#define DEF_COL_TAPIT	0xA00000
#define DEF_COL_TALAND	0x0000A0


Scale::Scale(QString iname)
{
	name=iname;
	policy = Auto;
	type = Linear;
	value = 256;
}

Scale::~Scale()
{
}

QPxSettings::QPxSettings()
{
#ifndef QT_NO_DEBUG
	qDebug() << QString("QPxSettings() %1").arg((unsigned int)this,8,16);
#endif
	geometry_mw = QRect(0,0,0,0);
	geometry_pref = QRect(0,0,0,0);
//	geometry_testsel = QRect(0,0,0,0);

	show_allctl = 0;

	useLocal  = 1;
	useRemote = 0;
	hosts.clear();
	ports.clear();

	for (int i=0; i<8; i++)	{
		col_errc.raw[i] = new QColor();
#ifndef QT_NO_DEBUG
//		qDebug(QString("new QColor %1").arg((unsigned int)col_errc.raw[i],8,16));
#endif
	}
}

QPxSettings::QPxSettings(const QPxSettings& o)
{
#ifndef QT_NO_DEBUG
	qDebug() << QString("QPxSettings(const QPxSettings&) %1").arg((unsigned int)this,8,16);
#endif
	for (int i=0; i<8; i++)	{ 
		col_errc.raw[i] = new QColor(*(o.col_errc.raw[i]));
#ifndef QT_NO_DEBUG
//		qDebug(QString("new QColor %1").arg((unsigned int)col_errc.raw[i],8,16));
#endif
	}
	*this = o;
}

QPxSettings& QPxSettings::operator = (const QPxSettings& o)
{
#ifndef QT_NO_DEBUG
	qDebug() << QString("QPxSettings() %1 = %2")
		.arg((unsigned int)this,8,16)
		.arg((unsigned int)&o,8,16);
#endif
	geometry_mw   = o.geometry_mw;
	geometry_pref = o.geometry_pref;

	show_allctl   = o.show_allctl;

	useLocal  = o.useLocal;
	useRemote = o.useRemote;
	hosts = o.hosts;
	ports = o.ports;

	tests		= o.tests;

	col_bg		= o.col_bg;
	col_bginv	= o.col_bginv;
	col_grid	= o.col_grid;	
	col_rspeed	= o.col_rspeed;
	col_wspeed	= o.col_wspeed;

	for (int i=0; i<8; i++)	*col_errc.raw[i] = *o.col_errc.raw[i];

	col_jitter	= o.col_jitter;
	col_asymm	= o.col_asymm;
	col_fe		= o.col_fe;
	col_te		= o.col_te;
	col_tapit	= o.col_tapit;
	col_taland	= o.col_taland;

	scales		= o.scales;
	return *this;
}

QPxSettings::~QPxSettings()
{
#ifndef QT_NO_DEBUG
	qDebug() << QString("~QPxSettings() %1").arg((unsigned int)this,8,16);
#endif
	for (int i=0; i<8; i++) {
#ifndef QT_NO_DEBUG
//		qDebug(QString("del QColor %1").arg((unsigned int)col_errc.raw[i],8,16));
#endif
		delete col_errc.raw[i];
	}
}

void QPxSettings::load()
{
	QSettings *settings;
#ifndef QT_NO_DEBUG
	qDebug("Loading settings...");
#endif

//	settings = new QSettings(QSettings::NativeFormat, QSettings::UserScope, "qpxtool");
	settings = new QSettings(QSettings::NativeFormat, QSettings::UserScope, "QPxTool", "qpxtool");
	settings->beginGroup("/general");
		show_allctl      = settings->value("show_allctl", 0).toBool();
	settings->endGroup();

	settings->beginGroup("/geometry");
		geometry_mw      = settings->value("mainwindow",  QRect(0,0,0,0) ).toRect();
		geometry_pref    = settings->value("preferences", QRect(0,0,0,0) ).toRect();
//		geometry_testsel = settings->value("testselector",  QRect(0,0,0,0) ).toRect();
	settings->endGroup();

	settings->beginGroup("/devices");
		useLocal  = settings->value("local", 1).toBool();
		useRemote = settings->value("remote", 0).toBool();
		hosts     = settings->value("hosts", QStringList()).toStringList();
		ports     = settings->value("ports", QStringList()).toStringList();
		hosts.removeAll("");
		tests	  = settings->value("tests", 0).toInt();
	settings->endGroup();

	settings->beginGroup("/colors");
		col_bg     = settings->value("graph_bg",   DEF_COL_BG).toInt();
		col_bginv  = QColor( (~col_bg.red()) & 0xFF, (~col_bg.green()) & 0xFF, (~col_bg.blue()) & 0xFF );
		col_grid   = settings->value("graph_grid", DEF_COL_GRID).toInt();
	
		col_rspeed = settings->value("rspeed",   DEF_COL_RSPEED).toInt();
		col_wspeed = settings->value("wspeed",   DEF_COL_WSPEED).toInt();

		for(int i=0; i<8; i++)
			*col_errc.raw[i] = settings->value(QString("errc%1").arg(i), DEF_COL_ERRC[i]).toInt();

		col_jitter = settings->value("jitter",	 DEF_COL_JITTER).toInt();
		col_asymm  = settings->value("asymm",	 DEF_COL_ASYMM).toInt();

		col_fe	   = settings->value("fe",		 DEF_COL_FE).toInt();
		col_te	   = settings->value("te",		 DEF_COL_TE).toInt();

		col_tapit  = settings->value("ta_pit",	 DEF_COL_TAPIT).toInt();
		col_taland = settings->value("ta_land",	 DEF_COL_TALAND).toInt();
	settings->endGroup();
	delete settings;
}

void QPxSettings::save()
{
	QSettings *settings;
#ifndef QT_NO_DEBUG
	qDebug("Saving settings...");
#endif
	settings = new QSettings(QSettings::NativeFormat, QSettings::UserScope, "QPxTool", "qpxtool");
	settings->beginGroup("/general");
		settings->setValue("show_allctl", show_allctl);
	settings->endGroup();
	settings->beginGroup("/geometry");
		settings->setValue("mainwindow",  geometry_mw);
		settings->setValue("preferences", geometry_pref);
//		settings->setValue("testselector", geometry_testsel);
	settings->endGroup();

	settings->beginGroup("/devices");
		settings->setValue("local",  useLocal);
		settings->setValue("remote", useRemote);
		settings->setValue("hosts",  hosts);
		settings->setValue("ports",  ports);
		settings->setValue("tests", tests);
	settings->endGroup();

	settings->beginGroup("/colors");
		settings->setValue("graph_bg",   col_bg.rgb());
		settings->setValue("graph_grid", col_grid.rgb());

		settings->setValue("rspeed",  	col_rspeed.rgb());
		settings->setValue("wspeed",  	col_wspeed.rgb());

		for(int i=0; i<8; i++)
			settings->setValue(QString("errc%1").arg(i), col_errc.raw[i]->rgb());

		settings->setValue("jitter",	col_jitter.rgb());
		settings->setValue("asymm",		col_asymm.rgb());

		settings->setValue("fe",		col_fe.rgb());
		settings->setValue("te",		col_te.rgb());

		settings->setValue("ta_pit",	col_tapit.rgb());
		settings->setValue("ta_land",	col_taland.rgb());

	settings->endGroup();

	delete settings;
	saveScale();
}

void QPxSettings::loadScale(QString name)
{
	QSettings *settings;
//	Scale scale(name);

#ifndef QT_NO_DEBUG
	qDebug() << "Loading scale: " << name;
#endif
//	settings = new QSettings(QSettings::NativeFormat, QSettings::UserScope, "qpxtool");
	settings = new QSettings(QSettings::NativeFormat, QSettings::UserScope, "QPxTool", "qpxtool");

	settings->beginGroup("/scale");
		settings->beginGroup(name);
			scales.get(name).policy = settings->value("policy", Scale::Auto).toInt();
			scales.get(name).type   = settings->value("type",   Scale::Log).toInt();
			scales.get(name).value  = settings->value("value", 0).toInt();
		settings->endGroup();
	settings->endGroup();

//	scales.removeAll(name);
//	scales.append(scale);

	delete settings;
}

void QPxSettings::saveScale()
{
	QSettings *settings;
#ifndef QT_NO_DEBUG
	qDebug("Saving scale...");
#endif
	settings = new QSettings(QSettings::NativeFormat, QSettings::UserScope, "QPxTool", "qpxtool");

	settings->beginGroup("/scale");
	for (int i=0; i<scales.size(); i++) {
#ifndef QT_NO_DEBUG
		qDebug() << scales[i].name;
#endif
		settings->beginGroup(scales[i].name);
			settings->setValue("policy", scales[i].policy);
			settings->setValue("type",   scales[i].type);
			settings->setValue("value",  scales[i].value);
		settings->endGroup();
	}
	settings->endGroup();

	delete settings;
}

void QPxSettings::setDefaultColors()
{
	col_bg     = DEF_COL_BG;
	col_bginv  = QColor( (~col_bg.red()) & 0xFF, (~col_bg.green()) & 0xFF, (~col_bg.blue()) & 0xFF );
	col_grid   = DEF_COL_GRID;
	
	col_rspeed = DEF_COL_RSPEED;
	col_wspeed = DEF_COL_WSPEED;

	for(int i=0; i<8; i++) *col_errc.raw[i] = DEF_COL_ERRC[i];

	col_jitter = DEF_COL_JITTER;
	col_asymm  = DEF_COL_ASYMM;

	col_fe	   = DEF_COL_FE;
	col_te	   = DEF_COL_TE;

	col_tapit  = DEF_COL_TAPIT;
	col_taland = DEF_COL_TALAND;
}

#if 0
QPxSettings& QPxSettings::operator = (const QPxSettings& o) {
	hosts = o.hosts;
	wgeometry = o.wgeometry;
	pgeometry = o.pgeometry;

	col_bg   = o.col_bg;
	col_grid = o.col_grid;

	scales   = o.scales;
	return (*this);
}
#endif

