/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2005-2009, Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include <stdio.h>
#include <string.h>

#include "common_functions.h"

int32_t __swap32(char* c) {
	char d[4];
	d[0]=c[3];
	d[1]=c[2];
	d[2]=c[1];
	d[3]=c[0];
	return *(int32_t*)d;
}

uint32_t __swap32u(char* c) {
	char d[4];
	d[0]=c[3];
	d[1]=c[2];
	d[2]=c[1];
	d[3]=c[0];
	return *(uint32_t*)d;
}

int32_t __swapw32(char* c) {
	char d[4];
	d[0]=c[2];
	d[1]=c[3];
	d[2]=c[0];
	d[3]=c[1];
	return *((int32_t*)d);
}

uint32_t __swapw32u(char* c) {
	char d[4];
	d[0]=c[2];
	d[1]=c[3];
	d[2]=c[0];
	d[3]=c[1];
	return *((uint32_t*)(char*)d);
}

int32_t swap32(char* c)			{ return __swap32(c); };
int32_t swap32(unsigned char* c){ return __swap32((char*)c); };
int32_t swap32(int32_t c)		{ return __swap32((char*)&c); }
int32_t swap32(uint32_t c)		{ return __swap32((char*)&c); }

uint32_t swap32u(char* c)		{ return __swap32u(c); };
uint32_t swap32u(unsigned char* c) { return __swap32u((char*)c); };
uint32_t swap32u(int32_t c)		{ return __swap32u((char*)&c); }
uint32_t swap32u(uint32_t c)	{ return __swap32u((char*)&c); }


int32_t swapw32(char* c)		 { return __swapw32(c); };
int32_t swapw32(unsigned char* c){ return __swapw32((char*)c); };
int32_t swapw32(int32_t c)		 { return __swapw32((char*)&c); }
int32_t swapw32(uint32_t c)		 { return __swapw32((char*)&c); }

uint32_t swapw32u(char* c)		 { return __swapw32(c); };
uint32_t swapw32u(unsigned char* c) { return __swapw32((char*)c); };
uint32_t swapw32u(int32_t c)	 { return __swapw32((char*)&c); }
uint32_t swapw32u(uint32_t c)	 { return __swapw32((char*)&c); }


int16_t __swap16(char* c) {
	char d[2];
	d[0]=c[1];
	d[1]=c[0];
	return *(short int*)d;
}

uint16_t __swap16u(char* c) {
	char d[2];
	d[0]=c[1];
	d[1]=c[0];
	return *(uint16_t*)d;
}

int16_t  swap16(char* c)		  { return __swap16(c); }
int16_t  swap16(unsigned char* c) { return __swap16((char*)c); }
int16_t  swap16(int16_t c)		  { return __swap16((char*)&c); }
int16_t  swap16(uint16_t c)		  { return __swap16((char*)&c); }
uint16_t swap16u(char* c)		  { return __swap16u(c); }
uint16_t swap16u(unsigned char* c) { return __swap16u((char*)c); }
uint16_t swap16u(int16_t c)		  { return __swap16((char*)&c); }
uint16_t swap16u(uint16_t c)	  { return __swap16((char*)&c); }


int32_t  to32(char* c)			 { return *(int32_t*)c; };
int32_t  to32(unsigned char* c)  { return *(int32_t*)c; };
int32_t  to32(int32_t c)		 { return c; }
int32_t  to32(uint32_t c)		 { return (int32_t)c; }

uint32_t to32u(char* c)			 { return *(uint32_t*)c; };
uint32_t to32u(unsigned char* c) { return *(uint32_t*)c; };
uint32_t to32u(int32_t c)		 { return *(uint32_t*)c; }
uint32_t to32u(uint32_t c)		 { return c; }

int16_t  to16(char* c)			 { return *(uint16_t*)c; };
int16_t  to16(unsigned char* c)  { return *(uint16_t*)c; };
uint16_t to16u(char* c)			 { return *(uint16_t*)c; };
uint16_t to16u(unsigned char* c) { return *(uint16_t*)c; };


void lba2msf(int lba, msf* time){
	time->m = lba/4500;
	time->s = (lba/75)%60;
	time->f = lba % 75;
}

int msf2lba(msf time){
	return time.m * 4500 + time.s * 75 + time.f;
}

void int2hms(int intt, hms* time){
	time->h = intt/3600;
	time->m = (intt/60)%60;
	time->s = intt % 60;
}

#ifndef _WIN32

int min(int a, int b)
{
	if (a<=b)
		return a;
	return b;
}

int max(int a, int b)
{
	if (a>b)
		return a;
	return b;
}

#else

int inet_aton(const char *cp, struct in_addr *addr)
{
	addr->s_addr = inet_addr(cp);
	return 1;
}

#endif

void remove_double_spaces(char* str)
{
	int len = strlen(str);
	while(len--) {
		if (*str == 0x20 && str[1] == 0x20) 
			strcpy(str, str+1);
		else
			str++;
	}
}

void remove_end_spaces(char* str)
{
	int len = strlen(str);
	while (str[--len] == 0x20)
		str[len] = 0;

}

int dispers(int m, int* arr, int l) {
	if ((!m) || (!arr) || (l<2)) return 0;
	long disp = 0;
	int i;
	int q;
	for (i=0; i<l; i++) {
	    q = (m - arr[i]);
	    disp += (q*q);
	}
	return (disp/(l-1));
}

