/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2005-2009 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#ifndef __common_functions_h
#define __common_functions_h

//#warning "COMMON_FUNCTIONS_H"

#include <inttypes.h>

#if defined(__unix) || defined(__unix__)

//#warning "UNIX"

#if defined(__APPLE__) && defined(__MACH__)
#include <machine/endian.h>
#else
#include <endian.h>
#endif

#include <arpa/inet.h>
#elif defined(_WIN32)
#include <sys/param.h>
#include <winsock2.h>
#ifndef _SOCKLEN_T
typedef int socklen_t;
#define EADDRINUSE		WSAEADDRINUSE
#define ECONNABORTED	WSAECONNABORTED

#define SHUT_RD			SD_RECEIVE
#define SHUT_WR			SD_SEND
#define SHUT_RDWR		SD_BOTH

#endif
#endif

typedef struct{
	int	m;
	int	s;
	int	f;
} msf;

typedef struct{
	int	h;
	int	m;
	int	s;
} hms;

extern int32_t  swap32(char* c);
extern int32_t  swap32(unsigned char* c);
extern int32_t  swap32(int32_t c_);
extern int32_t  swap32(uint32_t c_);
extern uint32_t swap32u(char* c);
extern uint32_t swap32u(unsigned char* c);
extern uint32_t swap32u(int32_t c_);
extern uint32_t swap32u(uint32_t c_);

extern int32_t  swapw32(char* c);
extern int32_t  swapw32(unsigned char* c);
extern int32_t  swapw32(int32_t c_);
extern int32_t  swapw32(uint32_t c_);
extern uint32_t swapw32u(char* c);
extern uint32_t swapw32u(unsigned char* c);
extern uint32_t swapw32u(int32_t c_);
extern uint32_t swapw32u(uint32_t c_);

extern int16_t  swap16(char* c);
extern int16_t  swap16(unsigned char* c);
extern int16_t  swap16(int16_t c);
extern int16_t  swap16(uint16_t c);
extern uint16_t swap16u(char* c);
extern uint16_t swap16u(unsigned char* c);
extern uint16_t swap16u(int16_t c);
extern uint16_t swap16u(uint16_t c);


extern int32_t  to32(char* c);
extern int32_t  to32(unsigned char* c);
extern int32_t  to32(int32_t c_);
extern int32_t  to32(uint32_t c_);

extern uint32_t to32u(char* c);
extern uint32_t to32u(unsigned char* c);
extern uint32_t to32u(int32_t c_);
extern uint32_t to32u(uint32_t c_);

extern int16_t  to16(char* c);
extern int16_t  to16(unsigned char* c);
extern uint16_t to16u(char* c);
extern uint16_t to16u(unsigned char* c);


//#if __BYTE_ORDER == __BIG_ENDIAN
#if BYTE_ORDER == BIG_ENDIAN

//#warning "BIG ENDIAN"
#define ntoh32(x)  to32(x)
#define ntoh32u(x) to32u(x)
#define ntoh16(x)  to16(x)
#define ntoh16u(x) to16u(x)

#define hton32(x)  to32(x)
#define hton32u(x) to32u(x)
#define hton16(x)  to16(x)
#define hton16u(x) to16u(x)

//#elif __BYTE_ORDER == __LITTLE_ENDIAN
#elif BYTE_ORDER == LITTLE_ENDIAN

//#warning "LITTLE ENDIAN"
#define ntoh32(x)  swap32(x)
#define ntoh32u(x) swap32u(x)
#define ntoh16(x)  swap16(x)
#define ntoh16u(x) swap16u(x)

#define hton32(x)  swap32(x)
#define hton32u(x) swap32u(x)
#define hton16(x)  swap16(x)
#define hton16u(x) swap16u(x)

//#elif __BYTE_ORDER == __PDP_ENDIAN
#elif BYTE_ORDER == PDP_ENDIAN

//#warning "PDP ENDIAN"
#define ntoh32(x)  swapw32(x)
#define ntoh32u(x) swapw32u(x)
#define ntoh16(x)  to16(x)
#define ntoh16u(x) to16u(x)

#define hton32(x)  swapw32(x)
#define hton32u(x) swapw32u(x)
#define hton16(x)  to16(x)
#define hton16u(x) to16u(x)

#endif


extern void lba2msf(int lba,  msf* time);
extern int  msf2lba(msf time);
extern void int2hms(int intt, hms* time);

#if defined (_WIN32)
	#include <stdlib.h>
	#define msleep(t) Sleep(t)
	#define sleep(t)  Sleep((t) << 10)

extern int inet_aton(const char *cp, struct in_addr *addr);

#else
	#define msleep(t) usleep((t) << 10)
extern int min(int a, int b);
extern int max(int a, int b);
#endif
extern void remove_double_spaces(char* str);
extern void remove_end_spaces(char* str);
//extern void file_path_name(char* str, char* fpath, char* fname);
//extern void file_suf_rm(char* str);
extern int dispers(int m, int* arr, int l);

#endif

