/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2007-2009, Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 *
 */

#include <stdio.h>
#include <qscan_plugin.h>

scan_plugin*	plugin_create(drive_info* idev)
{
	return new scan_benqrom(idev);
}

void plugin_destroy(scan_plugin* iplugin)
{
	if (iplugin != NULL) delete iplugin;
}

scan_benqrom::scan_benqrom(drive_info* idev)
{
	dev = idev;
	if (!dev->silent) printf("scan_benqrom()\n");
//	devlist = (drivedesclist*) &drivelist; 
	devlist = (drivedesc*) &drivelist; 
	test=0;
}

scan_benqrom::~scan_benqrom()
{
	if (!dev->silent) printf("~scan_benqrom()\n");
}

/*
int  scan_benqrom::probe_drive() {
	return DEV_FAIL;
}
*/

int  scan_benqrom::check_test(unsigned int itest)
{
	switch (itest) {
		case CHK_ERRC:
			if (dev->media.type & ~DISC_DVDRAM)
				return 0;
		default:
			return -1;
	}
	return -1;
}

int  scan_benqrom::start_test(unsigned int itest, long ilba)
{
	int r=-1;
	switch (itest) {
		case CHK_CD_ERRC:
		case CHK_CD_JB:
		case CHK_DVD_ERRC:
		case CHK_DVD_JB:
			lba=ilba;
			r = cmd_scan_init();
			break;
		default:
			return -1;
	}
	if (!r) {
		test = itest;
		return r;
	} else {
		test = 0;
		return r;
	}
}

int  scan_benqrom::scan_block(void *data, long *ilba)
{
	int r=-1;
	switch (test) {
		case CHK_CD_ERRC:
			r = cmd_cd_errc_block((cd_errc*)data);
			if(ilba) *ilba = lba;
			return r;
		case CHK_CD_JB:
			r = cmd_cd_jb_block((cdvd_jb*)data);
			if(ilba) *ilba = lba;
			return r;
		case CHK_DVD_ERRC:
			r = cmd_dvd_errc_block((dvd_errc*)data);
			if(ilba) *ilba = lba;
			return r;
		case CHK_DVD_JB:
			r = cmd_dvd_jb_block((cdvd_jb*)data);
			if(ilba) *ilba = lba;
			return r;
		default:
			return -1;
	}
}

int  scan_benqrom::end_test()
{ 
	switch (test) {
		case CHK_CD_ERRC:
		case CHK_CD_JB:
			cmd_cd_end();
			break;
		case CHK_DVD_ERRC:
		case CHK_DVD_JB:
			cmd_dvd_end();
			break;
		default:
			break;
	}
//	cmd_scan_end();
	test=0;
	return 0;
}

/*
__attribute__((constructor)) void init() {
    printf("init()\n");
}

__attribute__((destructor)) void exit() {
    printf("exit()\n");
}
*/

