/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2007-2008, Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 *
 */

#include <stdio.h>
#include <qscan_plugin.h>

scan_plugin*	plugin_create(drive_info* idev){
	return new scan_liteon(idev);
}

void plugin_destroy(scan_plugin* iplugin){
	if (iplugin != NULL) delete iplugin;
}

scan_liteon::scan_liteon(drive_info* idev) {
	dev = idev;
	if (!dev->silent) printf("scan_liteon()\n");
//	devlist = (drivedesclist*) &drivelist; 
	devlist = (drivedesc*) &drivelist; 
	test=0;
}

scan_liteon::~scan_liteon() {
	if (!dev->silent) printf("~scan_liteon()\n");
}

/*
int  scan_liteon::probe_drive() {
	return DEV_FAIL;
}
*/

int  scan_liteon::check_test(unsigned int itest)
{
	switch (itest) {
		case CHK_ERRC:
			if (dev->media.type & ~DISC_DVDRAM)
				return 0;
		default:
			return -1;
	}
	return -1;
}

int  scan_liteon::start_test(unsigned int itest, long ilba){
	int r=-1;
	switch (itest) {
		case CHK_CD_ERRC:
			lba=ilba;
			r = cmd_cd_errc_init();
			break;
		case CHK_DVD_ERRC:
			lba=ilba;
			r = cmd_dvd_errc_init();
			break;
		default:
			return -1;
	}
	if (!r) {
		test = itest;
		return r;
	} else {
		end_test();
		return r;
	}
}

int  scan_liteon::scan_block(void *data, long *ilba) {
	int r=-1;
	switch (test) {
		case CHK_CD_ERRC:
			r = cmd_cd_errc_block((cd_errc*)data);
			if(ilba) *ilba = lba;
			return r;
		case CHK_DVD_ERRC:
			r = cmd_dvd_errc_block((dvd_errc*)data);
			if(ilba) *ilba = lba;
			return r;
		default:
			return -1;
	}
}

int  scan_liteon::end_test() {
	switch (test) {
		case CHK_CD_ERRC:
			cmd_cd_errc_end();
			break;
		case CHK_DVD_ERRC:
			cmd_dvd_errc_end();
			break;
		default:
			break;
	}
	test=0;
	return 0;
}

/*
__attribute__((constructor)) void init() {
    printf("init()\n");
}

__attribute__((destructor)) void exit() {
    printf("exit()\n");
}
*/

