/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2007-2009, Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 *
 */

#include <stdio.h>
#include <qscan_plugin.h>

scan_plugin*	plugin_create(drive_info* idev)
{
	return new scan_plextor(idev);
}

void plugin_destroy(scan_plugin* iplugin)
{
	if (iplugin != NULL) delete iplugin;
}

scan_plextor::scan_plextor(drive_info* idev)
{
	dev = idev;
	if (!dev->silent) printf("scan_plextor()\n");
	devlist = (drivedesc*) &drivelist;
	test=0;
}

scan_plextor::~scan_plextor()
{
	if (!dev->silent) printf("~scan_plextor()\n");
}

/*
int  scan_plextor::probe_dev() {
	return DEV_FAIL;
}
*/

int  scan_plextor::check_test(unsigned int itest)
{
	switch (itest) {
		case CHK_ERRC:
		case CHK_JB:
			if (dev->media.type & ~DISC_DVDRAM)
				return 0;
		case CHK_FETE:
			if (dev->media.type & (DISC_CD | DISC_DVD) & ~DISC_CDROM & ~DISC_DVDROM)
				return 0;
		case CHK_TA:
			if (dev->media.type & (DISC_DVD & ~DISC_DVDROM))
			//if (dev->media.type & DISC_DVD)
				return 0;
		default:
			return -1;
	}
	return -1;
}

int  scan_plextor::start_test(unsigned int itest, long ilba)
{
	int r=-1;
	plextor_px755_do_auth(dev);
	switch (itest) {
		case CHK_CD_ERRC:
//		case CHK_CD_ERRCEX:
			lba=ilba;
			r = cmd_cd_errc_init();
			break;
		case CHK_CD_JB:
			lba=ilba;
			r = cmd_cd_jb_init();
			break;
		case CHK_DVD_ERRC:
//		case CHK_DVD_ERRCEX:
			lba=ilba;
			r = cmd_dvd_errc_init();
			break;
		case CHK_DVD_JB:
			lba=ilba;
			r = cmd_dvd_jb_init();
			break;
#if 1
		case CHK_FETE:
#else
		case CHK_FETE_CD:
		case CHK_FETE_CDROM:
		case CHK_FETE_DVD:
		case CHK_FETE_DVDROM:
#endif
			lba=ilba;
			r = cmd_fete_init();
			break;
		case CHK_TA:
			if (dev->media.type & (DISC_DVD & ~DISC_DVDROM)) {
				r = 0;
				test = CHK_TA_DVD;
				dev->parms.read_speed_kb = 2 * 1385;
				set_rw_speeds(dev);
				return 0;
			}
			break;
		default:
			return -1;
	}
	if (!r) {
		test = itest;
		return 0;
	} else {
		test = 0;
		return r;
	}
}

int  scan_plextor::scan_block(void *data, long *ilba)
{
	int r=-1;
	switch (test) {
		case CHK_CD_ERRC:
//		case CHK_CD_ERRCEX:
			r = cmd_cd_errc_block((cd_errc*)data);
			if(ilba) *ilba = lba;
			return r;
		case CHK_CD_JB:
			r = cmd_cd_jb_block((cdvd_jb*)data);
			if(ilba) *ilba = lba;
			return r;
		case CHK_DVD_ERRC:
//		case CHK_DVD_ERRCEX:
			r = cmd_dvd_errc_block((dvd_errc*)data);
			if(ilba) *ilba = lba;
			return r;
		case CHK_DVD_JB:
			r = cmd_dvd_jb_block((cdvd_jb*)data);
			if(ilba) *ilba = lba;
			return r;
#if 1
		case CHK_FETE:
#else
		case CHK_FETE_CD:
		case CHK_FETE_CDROM:
		case CHK_FETE_DVD:
		case CHK_FETE_DVDROM:
#endif
			r = cmd_fete_block((cdvd_ft*)data);
			if(ilba) *ilba = lba;
			return r;
		case CHK_TA_DVD:
			cmd_dvd_ta_block((cdvd_ta*)data);
			return r;
		default:
			return -1;
	}
}

int  scan_plextor::end_test()
{
	switch (test) {
		case CHK_CD_ERRC:
		case CHK_DVD_ERRC:
		case CHK_CD_JB:
		case CHK_DVD_JB:
			cmd_scan_end();
			break;
#if 1
		case CHK_FETE:
#else
		case CHK_FETE_CD:
		case CHK_FETE_CDROM:
		case CHK_FETE_DVD:
		case CHK_FETE_DVDROM:
#endif
			cmd_fete_end();
			break;
		case CHK_TA:
		case CHK_TA_CD:
		case CHK_TA_CDROM:
		case CHK_TA_DVD:
		case CHK_TA_DVDROM:
			break;
		default:
			break;
	}
	test=0;
	return 0;
}

/*
__attribute__((constructor)) void init() {
    printf("init()\n");
}

__attribute__((destructor)) void exit() {
    printf("exit()\n");
}
*/

